import os
from nose.plugins.base import Plugin

import tempfile
import difflib
import shutil
import errno

class NoseTestDiff( Plugin ):
    """
    Failed test diff output plugin for nose.
    """

    def help(self):
        return "Failed test diff report of test status into reportfile (specifiable with --failed-test-file)"

    def add_options(self, parser, env=os.environ):
        Plugin.add_options(self, parser, env)
        parser.add_option("--failed-test-file", action="store", default="failed_tests.txt", dest="failed_test_file", help="File to output failed tests report to")
    
    def configure(self, options, config):
        Plugin.configure(self, options, config)
        self.conf = config
        self.failed_test_fname = options.failed_test_file
        self.temp = tempfile.NamedTemporaryFile( 'w' )

    def finalize(self, result):
        current = '.'.join( ( self.failed_test_fname, 'current' ) )
        previous = '.'.join( ( self.failed_test_fname, 'previous' ) )
        if not os.path.exists( current ):
            open( current, 'w' )
        os.rename( current, previous )
        shutil.copy( self.temp.name, current )
        self.temp.close()
        failed_test_file = open( self.failed_test_fname, 'w' )
        diff = list( difflib.Differ().compare( open( previous, 'r' ).readlines(), open( current, 'r' ).readlines() ) )
        if diff:
            print >> failed_test_file, 'Failed Tests:'
        for line in diff:
            if line.startswith( '?' ):
                continue
            print >> failed_test_file, line,
        try:
            failed_test_file.close()
        except IOError, e:
            if e.errno != errno.EINVAL:
                raise

    def addFailure(self, test, error ):
        """
        Test failed
        """
        print >> self.temp, str( test.shortDescription() )
        self.temp.flush()

    def addError(self, test, error ):
        """
        Test errored.
        """
        print >> self.temp, "%s, %s" % ( test.id(), test.shortDescription() )
        self.temp.flush()
